@@ -10,7 +10,7 @@
  *
  * The rest of the file is licensed under the BSD license.  See LICENSE.
  *
- * $Id: assoc.c,v 1.5 2003/06/27 21:15:55 bradfitz Exp $
+ * $Id: assoc.c,v 1.6 2003/08/11 05:44:26 bradfitz Exp $
  */
 
 #include <sys/types.h>
@@ -19,7 +19,6 @@
 #include <sys/socket.h>
 #include <sys/signal.h>
 #include <sys/resource.h>
-#include <sys/mman.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -28,7 +27,6 @@
 #include <netinet/in.h>
 #include <errno.h>
 #include <event.h>
-#include <malloc.h>
 #include <assert.h>
 
 #include "memcached.h"
