@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*- */
-/* $Id: items.c,v 1.21 2004/03/31 05:01:51 avva Exp $ */
+/* $Id: items.c,v 1.23 2004/09/13 22:31:53 avva Exp $ */
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -214,7 +214,7 @@ char *item_cachedump(unsigned int slabs_clsid, unsigned int limit, unsigned int
     item *it;
     int len;
     int shown = 0;
-    char temp[256];
+    char temp[512];
     
     if (slabs_clsid > LARGEST_ID) return 0;
     it = heads[slabs_clsid];
@@ -223,14 +223,9 @@ char *item_cachedump(unsigned int slabs_clsid, unsigned int limit, unsigned int
     if (buffer == 0) return 0;
     bufcurr = 0;
 
-    while(1) {
-        if(limit && shown >=limit)
-            break;
-        if (!it)
-            break;
-        sprintf(temp, "ITEM %s [%u b; %lu s]\r\n", ITEM_key(it), it->nbytes - 2, it->time);
-        len = strlen(temp);
-        if (bufcurr + len +5 > memlimit)  /* 5 is END\r\n */
+    while (it && (!limit || shown < limit)) {
+        len = sprintf(temp, "ITEM %s [%u b; %lu s]\r\n", ITEM_key(it), it->nbytes - 2, it->time);
+        if (bufcurr + len + 6 > memlimit)  /* 6 is END\r\n\0 */
             break;
         strcpy(buffer + bufcurr, temp);
         bufcurr+=len;
@@ -278,7 +273,7 @@ char* item_stats_sizes(int *bytes) {
     }
 
     /* build the histogram */
-    memset(buf, 0, num_buckets * sizeof(int));
+    memset(histogram, 0, num_buckets * sizeof(int));
     for (i=0; i<LARGEST_ID; i++) {
         item *iter = heads[i];
         while (iter) {
