@@ -13,7 +13,7 @@
  *      Anatoly Vorobey <mellon@pobox.com>
  *      Brad Fitzpatrick <brad@danga.com>
  *
- *  $Id: memcached.c,v 1.47 2004/04/24 20:00:59 avva Exp $
+ *  $Id: memcached.c,v 1.56 2005/04/05 00:10:26 bradfitz Exp $
  */
 
 #include "config.h"
@@ -173,7 +173,14 @@ conn *conn_new(int sfd, int init_state, int event_flags) {
     c->ev_flags = event_flags;
 
     if (event_add(&c->event, 0) == -1) {
-        free(c);
+        if (freecurr < freetotal) {
+            freeconns[freecurr++] = c;
+        } else {
+            free (c->rbuf);
+            free (c->wbuf);
+            free (c->ilist);
+            free (c);
+        }
         return 0;
     }
 
@@ -273,6 +280,12 @@ void complete_nread(conn *c) {
 
         old_it = assoc_find(ITEM_key(it));
 
+        if (old_it && settings.oldest_live &&
+            old_it->time <= settings.oldest_live) {
+            item_unlink(old_it);
+            old_it = 0;
+        }
+
         if (old_it && old_it->exptime && old_it->exptime < now) {
             item_unlink(old_it);
             old_it = 0;
@@ -321,10 +334,10 @@ void process_stat(conn *c, char *command) {
 
         pos += sprintf(pos, "STAT pid %u\r\n", pid);
         pos += sprintf(pos, "STAT uptime %lu\r\n", now - stats.started);
-        pos += sprintf(pos, "STAT time %u\r\n", now);
+        pos += sprintf(pos, "STAT time %ld\r\n", now);
         pos += sprintf(pos, "STAT version " VERSION "\r\n");
-        pos += sprintf(pos, "STAT rusage_user %u:%u\r\n", usage.ru_utime.tv_sec, usage.ru_utime.tv_usec);
-        pos += sprintf(pos, "STAT rusage_system %u:%u\r\n", usage.ru_stime.tv_sec, usage.ru_stime.tv_usec);
+        pos += sprintf(pos, "STAT rusage_user %ld.%06ld\r\n", usage.ru_utime.tv_sec, usage.ru_utime.tv_usec);
+        pos += sprintf(pos, "STAT rusage_system %ld.%06ld\r\n", usage.ru_stime.tv_sec, usage.ru_stime.tv_usec);
         pos += sprintf(pos, "STAT curr_items %u\r\n", stats.curr_items);
         pos += sprintf(pos, "STAT total_items %u\r\n", stats.total_items);
         pos += sprintf(pos, "STAT bytes %llu\r\n", stats.curr_bytes);
@@ -504,7 +517,7 @@ void process_command(conn *c, char *command) {
         int len, res;
         item *it;
 
-        res = sscanf(command, "%*s %250s %u %lu %d\n", key, &flags, &expire, &len);
+        res = sscanf(command, "%*s %250s %u %ld %d\n", key, &flags, &expire, &len);
         if (res!=4 || strlen(key)==0 ) {
             out_string(c, "CLIENT_ERROR bad command line format");
             return;
@@ -606,8 +619,9 @@ void process_command(conn *c, char *command) {
             if (it && (it->it_flags & ITEM_DELETED)) {
                 it = 0;
             }
-            if (settings.oldest_live && it && 
+            if (settings.oldest_live && it &&
                 it->time <= settings.oldest_live) {
+                item_unlink(it);
                 it = 0;
             }
             if (it && it->exptime && it->exptime < now) {
@@ -616,15 +630,18 @@ void process_command(conn *c, char *command) {
             }
 
             if (it) {
+                if (i >= c->isize) {
+                    item **new_list = realloc(c->ilist, sizeof(item *)*c->isize*2);
+                    if (new_list) {
+                        c->isize *= 2;
+                        c->ilist = new_list;
+                    } else break;
+                }
                 stats.get_hits++;
                 it->refcount++;
                 item_update(it);
                 *(c->ilist + i) = it;
                 i++;
-                if (i > c->isize) {
-                    c->isize *= 2;
-                    c->ilist = realloc(c->ilist, sizeof(item *)*c->isize);
-                }
             } else stats.get_misses++;
         }
         c->icurr = c->ilist;
@@ -646,13 +663,34 @@ void process_command(conn *c, char *command) {
         int res;
         time_t exptime = 0;
 
-        res = sscanf(command, "%*s %250s %d", key, &exptime);
+        res = sscanf(command, "%*s %250s %ld", key, &exptime);
         it = assoc_find(key);
         if (!it) {
             out_string(c, "NOT_FOUND");
             return;
         }
 
+        if (exptime == 0) {
+            item_unlink(it);
+            out_string(c, "DELETED");
+            return;
+        }
+
+        if (delcurr >= deltotal) {
+            item **new_delete = realloc(todelete, sizeof(item *) * deltotal * 2);
+            if (new_delete) {
+                todelete = new_delete;
+                deltotal *= 2;
+            } else { 
+                /* 
+                 * can't delete it immediately, user wants a delay,
+                 * but we ran out of memory for the delete queue
+                 */
+                out_string(c, "SERVER_ERROR out of memory");
+                return;
+            }
+        }
+            
         exptime = realtime(exptime);
 
         it->refcount++;
@@ -660,10 +698,6 @@ void process_command(conn *c, char *command) {
         it->exptime = exptime;
         it->it_flags |= ITEM_DELETED;
         todelete[delcurr++] = it;
-        if (delcurr >= deltotal) {
-            deltotal *= 2;
-            todelete = realloc(todelete, sizeof(item *)*deltotal);
-        }
         out_string(c, "DELETED");
         return;
     }
@@ -1035,11 +1069,10 @@ void drive_machine(conn *c) {
                 case 0:
                     it = *(c->icurr);
                     assert((it->it_flags & ITEM_SLABBED) == 0);
-                    sprintf(c->ibuf, "VALUE %s %u %u\r\n", ITEM_key(it), it->flags, it->nbytes - 2);
+                    c->ibytes = sprintf(c->ibuf, "VALUE %s %u %u\r\n", ITEM_key(it), it->flags, it->nbytes - 2);
                     if (settings.verbose > 1)
                         fprintf(stderr, ">%d sending key %s\n", c->sfd, ITEM_key(it));
                     c->iptr = c->ibuf;
-                    c->ibytes = strlen(c->iptr);
                     c->ipart = 1;
                     break;
                 case 3:
@@ -1117,6 +1150,12 @@ int server_socket(int port) {
     setsockopt(sfd, IPPROTO_TCP, TCP_NODELAY, &flags, sizeof(flags));
 #endif
 
+    /* 
+     * the memset call clears nonstandard fields in some impementations
+     * that otherwise mess things up.
+     */
+    memset(&addr, 0, sizeof(addr));
+
     addr.sin_family = AF_INET;
     addr.sin_port = htons(port);
     addr.sin_addr = settings.interface;
@@ -1145,8 +1184,16 @@ struct event deleteevent;
 
 void delete_handler(int fd, short which, void *arg) {
     struct timeval t;
+    static int initialized = 0;
+
+    if (initialized) {
+        /* some versions of libevent don't like deleting events that don't exist,
+           so only delete once we know this event has been added. */
+        evtimer_del(&deleteevent);
+    } else {
+        initialized = 1;
+    }
 
-    evtimer_del(&deleteevent);
     evtimer_set(&deleteevent, delete_handler, 0);
     t.tv_sec = 5; t.tv_usec=0;
     evtimer_add(&deleteevent, &t);
@@ -1186,6 +1233,7 @@ void usage(void) {
     printf("-vv           very verbose (also print client commands/reponses)\n");
     printf("-h            print this help and exit\n");
     printf("-i            print memcached and libevent license\n");
+    printf("-P <file>     save PID in <file>, only used with -d option\n");
     return;
 }
 
@@ -1260,6 +1308,33 @@ void usage_license(void) {
     return;
 }
 
+void save_pid(pid_t pid,char *pid_file) {
+    FILE *fp;
+    if (!pid_file)
+        return;
+
+    if (!(fp = fopen(pid_file,"w"))) {
+        fprintf(stderr,"Could not open the pid file %s for writing\n",pid_file);
+        return;
+    }
+
+    fprintf(fp,"%ld\n",(long) pid);
+    if (fclose(fp) == -1) {
+        fprintf(stderr,"Could not close the pid file %s.\n",pid_file);
+        return;
+    }
+}
+
+void remove_pidfile(char *pid_file) {
+  if (!pid_file)
+      return;
+
+  if (unlink(pid_file)) {
+      fprintf(stderr,"Could not remove the pid file %s.\n",pid_file);
+  }
+
+}
+
 int l_socket=0;
 
 int main (int argc, char **argv) {
@@ -1273,12 +1348,13 @@ int main (int argc, char **argv) {
     struct passwd *pw;
     struct sigaction sa;
     struct rlimit rlim;
+    char *pid_file = NULL;
 
     /* init settings */
     settings_init();
 
     /* process arguments */
-    while ((c = getopt(argc, argv, "p:m:Mc:khirvdl:u:")) != -1) {
+    while ((c = getopt(argc, argv, "p:m:Mc:khirvdl:u:P:")) != -1) {
         switch (c) {
         case 'p':
             settings.port = atoi(optarg);
@@ -1321,6 +1397,9 @@ int main (int argc, char **argv) {
         case 'u':
             username = optarg;
             break;
+        case 'P':
+            pid_file = optarg;
+            break;
         default:
             fprintf(stderr, "Illegal argument \"%c\"\n", c);
             return 1;
@@ -1456,9 +1535,17 @@ int main (int argc, char **argv) {
     todelete = malloc(sizeof(item *)*deltotal);
     delete_handler(0,0,0); /* sets up the event */
 
+    /* save the PID in if we're a daemon */
+    if (daemonize)
+        save_pid(getpid(),pid_file);
+
     /* enter the loop */
     event_loop(0);
 
+    /* remove the PID file if we're a daemon */
+    if (daemonize)
+        remove_pidfile(pid_file);
+
     return 0;
 }
 
